﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;
using Verse.AI;

namespace RimWorld
{
    public class JobDriver_LoadShipBuilding : JobDriver
	{
		private const TargetIndex RefuelableInd = TargetIndex.A;
		private const TargetIndex FuelInd = TargetIndex.B;
		public const int Duration = 240;

		protected Thing Refuelable
		{
			get
			{
				return this.job.GetTarget(TargetIndex.A).Thing;
			}
		}

		protected CompRefuelable RefuelableComp
		{
			get
			{
				return this.Refuelable.TryGetComp<CompRefuelable>();
			}
		}

		protected Thing Fuel
		{
			get
			{
				return this.job.GetTarget(TargetIndex.B).Thing;
			}
		}

		public override bool TryMakePreToilReservations(bool errorOnFailed)
		{
			return this.pawn.Reserve(this.Refuelable, this.job, 1, -1, null, errorOnFailed) && this.pawn.Reserve(this.Fuel, this.job, 1, -1, null, errorOnFailed);
		}

		protected override IEnumerable<Toil> MakeNewToils()
		{
			this.job.count = 1;
			ToilFailConditions.FailOnDespawnedNullOrForbidden(this, TargetIndex.A);
			ToilFailConditions.FailOnBurningImmobile(this, TargetIndex.A);
			yield return Toils_Reserve.Reserve(TargetIndex.A, 1, 1, null);
			Toil reserveFuel = Toils_Reserve.Reserve(TargetIndex.B, 1, 1, null);
			yield return reserveFuel;
			yield return ToilFailConditions.FailOnSomeonePhysicallyInteracting(ToilFailConditions.FailOnDespawnedNullOrForbidden(Toils_Goto.GotoThing(TargetIndex.B, PathEndMode.ClosestTouch), TargetIndex.B), TargetIndex.B);
			yield return ToilFailConditions.FailOnDestroyedNullOrForbidden(Toils_Haul.StartCarryThing(TargetIndex.B, false, true, false), TargetIndex.B);
			yield return Toils_Haul.CheckForGetOpportunityDuplicate(reserveFuel, TargetIndex.B, TargetIndex.None, true, null);
			yield return Toils_Goto.GotoThing(TargetIndex.A, PathEndMode.Touch);
			yield return ToilEffects.WithProgressBarToilDelay(ToilFailConditions.FailOnDestroyedNullOrForbidden(ToilFailConditions.FailOnDestroyedNullOrForbidden(Toils_General.Wait(Duration, TargetIndex.None), TargetIndex.B), TargetIndex.A), TargetIndex.A, false, -0.5f);
			Toil toil = new Toil();
			toil.initAction = delegate
			{
				RefuelableComp.Refuel(new List<Thing>() { Fuel });
			};
			toil.defaultCompleteMode = ToilCompleteMode.Instant;
			yield return toil;
			/*/
			this.FailOnDespawnedNullOrForbidden(TargetIndex.A);
			base.AddEndCondition(delegate
			{
				if (!this.RefuelableComp.IsFull)
				{
					return JobCondition.Ongoing;
				}
				return JobCondition.Succeeded;
			});
			yield return Toils_General.DoAtomic(delegate
			{
				this.job.count = this.RefuelableComp.GetFuelCountToFullyRefuel();
			});
			Toil reserveFuel = Toils_Reserve.Reserve(TargetIndex.B, 1, -1, null);
			yield return reserveFuel;
			yield return Toils_Goto.GotoThing(TargetIndex.B, PathEndMode.ClosestTouch).FailOnDespawnedNullOrForbidden(TargetIndex.B).FailOnSomeonePhysicallyInteracting(TargetIndex.B);
			yield return Toils_Haul.StartCarryThing(TargetIndex.B, false, true, false, true).FailOnDestroyedNullOrForbidden(TargetIndex.B);
			yield return Toils_Haul.CheckForGetOpportunityDuplicate(reserveFuel, TargetIndex.B, TargetIndex.None, true, null);
			yield return Toils_Goto.GotoThing(TargetIndex.A, PathEndMode.Touch);
			yield return Toils_General.Wait(Duration, TargetIndex.None).FailOnDestroyedNullOrForbidden(TargetIndex.B).FailOnDestroyedNullOrForbidden(TargetIndex.A).FailOnCannotTouch(TargetIndex.A, PathEndMode.Touch).WithProgressBarToilDelay(TargetIndex.A, false, -0.5f);
			yield return Toils_Refuel.FinalizeRefueling(TargetIndex.A, TargetIndex.B);
			yield break;*/
		}
	}
}
